#![allow(non_upper_case_globals)]
#![allow(non_camel_case_types)]
#![allow(non_snake_case)]
#![allow(unused)]

#[macro_use]
extern crate log;
#[macro_use]
extern crate simple_error;

mod bindings;
use bindings::{PredictorHandle, TreelitePredictorFree, TreelitePredictorLoad,  TreelitePredictorEntry, TreelitePredictorPredictInst};
use std::ffi::{CString,c_void, CStr};
use std::{ptr,error::Error};
use std::fs;
use std::path::PathBuf;

pub struct Predictor {
    ph: PredictorHandle,
}

impl Predictor {
    pub fn new(path: &str) -> Predictor {
        let mut p = Predictor {
            ph: ptr::null_mut(),
        };
        unsafe {
            let cpath = CString::new(path).expect(&format!("failed to load {}", path));
            let r = TreelitePredictorLoad(cpath.as_ptr(), 1, &mut p.ph);
            debug!("treelite lib {} loaded : {}", path, r == 0);
        }
        p
    }

    pub fn predict_inst(&self, input: &Vec<f32>)-> Result<f32, Box<dyn Error>>{
        unsafe{
            let mut pes = Vec::<TreelitePredictorEntry>::new();
            for item in input{
                pes.push(TreelitePredictorEntry{fvalue:*item});
            }
            let inputPtr: *mut TreelitePredictorEntry = pes.as_mut_ptr(); 
            let mut result:f32 = 0.0;
            let mut result_size:u64 = 0;
            let mut r = TreelitePredictorPredictInst(self.ph, inputPtr, 0 as std::os::raw::c_int, &mut result, &mut result_size);
            debug!("TreeLitePrediction op:{}, result:{}",r, result);
            match r{
                0 => Ok(result),
                _ => bail!("treelite predict_inst error"),
            }
        }
    }
}

impl Drop for Predictor {
    fn drop(&mut self) {
        unsafe {
            let r = TreelitePredictorFree(self.ph);
            debug!("treelite lib freed : {}", r == 0);
        }
    }
}

#[cfg(test)]
mod tests {
    use super::*;
    #[test]
    fn test_predictor() {
        env_logger::init();
        {
            let path = PathBuf::from("test/eta.so");
            let p = Predictor::new(&fs::canonicalize(&path).unwrap().into_os_string().into_string().unwrap());
            assert_eq!(p.predict_inst(vec![5.639,-0.280,5.648,-0.244,21.0,5.0,6507.0,884.0,4036.0]).unwrap(),1397.746);
        }
    }
}
